/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Alessandro Rubini <rubini@gnudd.com>
 *
 * This work is part of the White Rabbit project, a research effort led
 * by CERN, the European Institute for Nuclear Research.
 */
#ifndef __UAPI_LINUX_IPMI_FRU_H__
#define __UAPI_LINUX_IPMI_FRU_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __KERNEL__
#  include <linux/types.h>
#  include <linux/string.h>
#else
#  include <stdint.h>
#  include <string.h>
#endif

#define FRU_SIZE_MAX 256
/*
 * These structures match the unaligned crap we have in FRU1011.pdf
 * (http://download.intel.com/design/servers/ipmi/FRU1011.pdf)
 */

/**
 * struct fru_common_header - FRU header structure
 * @format: must be 0x01
 * @internal_use_off: for internal user - multiple of 8 bytes
 * @chassis_info_off: chassis info - multiple of 8 bytes
 * @board_area_off: board info area - multiple of 8 bytes
 * @product_area_off: product area -  multiple of 8 bytes
 * @multirecord_off: multirecord -  multiple of 8 bytes
 * @pad: must be 0
 * @checksum: sum modulo 256 must be 0
 *
 * FRU specification chapter 8, page 5
 */
struct fru_common_header {
	uint8_t format;
	uint8_t internal_use_off;
	uint8_t chassis_info_off;
	uint8_t board_area_off;
	uint8_t product_area_off;
	uint8_t multirecord_off;
	uint8_t pad;
	uint8_t checksum;
};

/* chapter 9, page 5 -- internal_use: not used by us */

/* chapter 10, page 6 -- chassis info: not used by us */

/**
 * struct fru_type_length
 * @type_length: type and length
 * @data: pointer to value
 *
 * FRU specification  chapter 13, page 9 -- used by board_info_area below
 */
struct fru_type_length {
	uint8_t type_length;
	uint8_t data[0];
};

/**
 * struct fru_board_info_area - Board Info Area Section
 * @format: must be 0x01
 * @area_len:  area length- multiple of 8 bytes
 * @language: I hope it's 0
 * @mfg_date: LSB, minutes since 1996-01-01
 * @tl:	type-length stuff follows
 *
 * FRU specification chapter 11, page 7
 */
struct fru_board_info_area {
	uint8_t format;
	uint8_t area_len;
	uint8_t language;
	uint8_t mfg_date[3];
	struct fru_type_length tl[0];

	/*
	 * the TL there are in order:
	 * Board Manufacturer
	 * Board Product Name
	 * Board Serial Number
	 * Board Part Number
	 * FRU File ID (may be null)
	 * more manufacturer-specific stuff
	 * 0xc1 as a terminator
	 * 0x00 pad to a multiple of 8 bytes - 1
	 * checksum (sum of all stuff module 256 must be zero)
	 */
};

/**
 * enum fru_type - List of possible FRU types
 * @FRU_TYPE_BINARY: binary value
 * @FRU_TYPE_BCDPLUS: BCD+ value
 * @FRU_TYPE_ASCII6: ascii value
 * @FRU_TYPE_ASCII: not ascii: depends on language
 */
enum fru_type {
	FRU_TYPE_BINARY		= 0x00,
	FRU_TYPE_BCDPLUS	= 0x40,
	FRU_TYPE_ASCII6		= 0x80,
	FRU_TYPE_ASCII		= 0xc0, /* not ascii: depends on language */
};

/**
 * fru_get_board_area() - Get the board info area
 * @header: FRU header pointer
 *
 * Return: pointer to the board info area
 */
static inline struct fru_board_info_area *fru_get_board_area(
	const struct fru_common_header *header)
{
	/* we know for sure that the header is 8 bytes in size */
	return (struct fru_board_info_area *)(header + header->board_area_off);
}

/**
 * fru_type() - Get the FRU section type
 * @tl: the type-length
 *
 * Return: FRU section type
 */
static inline int fru_type(struct fru_type_length *tl)
{
	return tl->type_length & 0xc0;
}

/**
 * fru_length() - Get the FRU section length
 * @tl: the type-length
 *
 * Return: FRU section length
 */
static inline int fru_length(struct fru_type_length *tl)
{
	return (tl->type_length & 0x3f) + 1; /* len of whole record */
}

/**
 * fru_strlen() - Compute the string value length from type-length
 * @tl: the type-length
 *
 * Return: string length
 *
 * assume ascii-latin1 encoding
 */
static inline int fru_strlen(struct fru_type_length *tl)
{
	return fru_length(tl) - 1;
}

/**
 * fru_strcpy() - Copy the string value from type-length
 * @dest: destination string buffer
 * @tl: the type-length
 *
 * Return: it returns the pointer to dest
 */
static inline char *fru_strcpy(char *dest, struct fru_type_length *tl)
{
	int len = fru_strlen(tl);
	memcpy(dest, tl->data, len);
	dest[len] = '\0';
	return dest;
}

/**
 * fru_next_tl() - Retrieve the next type-length structure
 * @tl: the current type-length
 *
 * Return: the next type-length structure
 */
static inline struct fru_type_length *fru_next_tl(struct fru_type_length *tl)
{
	return tl + fru_length(tl);
}

/**
 * fru_is_eof() - fru_is_eofCheck if there are no more type-length
 * @tl: the current type-length
 *
 * Return:
 * - 1 if the current type-length is the last one
 * - 0 if the current type-length is not the last one
 */
static inline int fru_is_eof(struct fru_type_length *tl)
{
	return tl->type_length == 0xc1;
}

/**
 * fru_header_cksum_ok() - Validate FRU header checksum
 * @header: FRU header pointer
 *
 * Return:
 * - 1 if the FRU header checksum is correct
 * - 0 if the FRU header checksum is not correct
 */
static inline int fru_header_cksum_ok(struct fru_common_header *header)
{
	uint8_t *ptr = (uint8_t *)header;
	int i, sum;

	for (i = sum = 0; i < sizeof(*header); i++)
		sum += ptr[i];

	return (sum & 0xff) == 0;
}

/**
 * fru_bia_cksum_ok() -Validate Board Info Area checksum
 * @bia: board info area pointer
 *
 * Return:
 * - 1 if the FRU board info area checksum is correct
 * - 0 if the FRU board info area checksum is not correct
 */
static inline int fru_bia_cksum_ok(struct fru_board_info_area *bia)
{
	uint8_t *ptr = (uint8_t *)bia;
	int i, sum;

	for (i = sum = 0; i < 8 * bia->area_len; i++)
		sum += ptr[i];
	return (sum & 0xff) == 0;
}

/* All these 4 return allocated strings by calling fru_alloc() */
extern char *fru_get_board_manufacturer(struct fru_common_header *header);
extern char *fru_get_product_name(struct fru_common_header *header);
extern char *fru_get_serial_number(struct fru_common_header *header);
extern char *fru_get_part_number(struct fru_common_header *header);

#ifdef __cplusplus
}
#endif

#endif /* __UAPI_LINUX_IPMI_FRU_H__ */
